--   This schema is used to initialize the in-memory database.

-- Some notes about the queue_state table
-- - There's one and only row.
-- - `paused` 0 = play, 1 = paused
-- - `random` whether the queue is played randomly
-- - `repeat` whether the queue loops
-- - `single` whether only one kara loops
-- - `current` the position in the queue of the kara being played
-- - `elapsed` the number of seconds from the beginning of the current kara
-- - `duration` the total duration of the playing kara

CREATE TABLE IF NOT EXISTS queue_state
  ( id          INTEGER PRIMARY KEY DEFAULT 42 CHECK(id = 42)
  , volume      INTEGER NOT NULL DEFAULT 100 CHECK(0 <= volume AND volume <= 100)
  , paused      INTEGER NOT NULL DEFAULT 1
  , random      INTEGER NOT NULL DEFAULT 0
  , repeat      INTEGER NOT NULL DEFAULT 0
  , single      INTEGER NOT NULL DEFAULT 0
  , consume     INTEGER NOT NULL DEFAULT 0
  , current     INTEGER CHECK(current > 0)
  , duration    INTEGER CHECK(duration >= 0)
  , elapsed     INTEGER CHECK(elapsed >= 0)
  );

INSERT INTO queue_state (id) VALUES (42);

-- Temporary queue table used when reordering the queue (for inserts)
CREATE TABLE IF NOT EXISTS queue_tmp
   ( position   INTEGER PRIMARY KEY AUTOINCREMENT CHECK(position > 0)
   , kara_id    INTEGER
   , priority   INTEGER NOT NULL DEFAULT 1 CHECK(priority > 0 AND priority < 6)
   );

-- Memory replica of the kara cache
CREATE TABLE IF NOT EXISTS kara_cache
  ( kara_id     INTEGER NOT NULL            -- Valid kara id, can't foreign key as the disk is not loaded
  , mtime       INTEGER NOT NULL DEFAULT 0  -- Last modification date
  , duration    INTEGER NOT NULL DEFAULT 0  -- Duration of the kara
  , file_path   TEXT    NOT NULL            -- Cached filepath
  , magic_error INTEGER NOT NULL DEFAULT 0  -- 1 if the mkv is valid, 0 otherwise
  );

-- Used to store the content of the ini configuration file.
CREATE TABLE IF NOT EXISTS config
  ( section     TEXT NOT NULL
  , key         TEXT NOT NULL
  , value       TEXT
  , PRIMARY KEY (section, key)
  ) WITHOUT ROWID;
