#if !defined(__LKT_THREAD_H__)
#define __LKT_THREAD_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <lektor/lktmodule.h>
#include <lektor/common.h>
#include <sys/types.h>

struct poller_thread {
    /* The input poll. */
    struct stack input;
    pthread_mutex_t input_lock;

    pthread_t th;
    unsigned int initial_size;

    /* The output pool. */
    struct stack output;
    pthread_mutex_t output_lock;
};

struct poller_thread_arg {
    struct poller_thread *self;
    void *args;
};

struct ___poller_thread_args {
    void *(*start)(struct poller_thread_arg *);
    struct poller_thread_arg *arg;
};

void *___poller_thread_start(void *args__);

UNUSED int poller_new(struct poller_thread *th, void *(*func)(struct poller_thread_arg *),
                      void *args);

UNUSED int poller_join(struct poller_thread *th, void **ret);

UNUSED int th_append(struct stack *lst, pthread_mutex_t *lock, void *ptr);

UNUSED int th_pop(struct stack *lst, pthread_mutex_t *lock, void **ptr);

UNUSED int th_head(struct stack *lst, pthread_mutex_t *lock, void **ptr);

#define __POLLER_ATOMICS_DEFINE(operation, inout, level)                                  \
    UNUSED static int poller_##operation##inout(struct poller_thread *th, void level ptr) \
    {                                                                                     \
        return th_##operation(&th->inout, &th->inout##_lock, ptr);                        \
    }
// clang-format off
__POLLER_ATOMICS_DEFINE(append, input,  *)
__POLLER_ATOMICS_DEFINE(append, output, *)
__POLLER_ATOMICS_DEFINE(pop,    input,  **)
__POLLER_ATOMICS_DEFINE(pop,    output, **)
__POLLER_ATOMICS_DEFINE(head,   input,  **)
__POLLER_ATOMICS_DEFINE(head,   output, **)
// clang-format on
#undef __POLLER_ATOMICS_DEFINE

#if defined(__cplusplus)
}
#endif
#endif /* __LKT_THREAD_H__ */
